@@ -33,9 +33,8 @@
 import os
 import sys
 import cgi
-from six.moves.urllib.parse import quote, unquote
-from six import StringIO
-from six.moves.configparser import SafeConfigParser
+from urllib.parse import quote, unquote
+from io import StringIO
 from pycsw.core.etree import etree
 from pycsw import oaipmh, opensearch, sru
 from pycsw.ogc.csw.cql import cql2fes1
@@ -713,8 +712,9 @@ class Csw2(object):
         if any(x in ['bbox', 'q', 'time'] for x in self.parent.kvp):
             LOGGER.debug('OpenSearch Geo/Time parameters detected.')
             self.parent.kvp['constraintlanguage'] = 'FILTER'
-            tmp_filter = opensearch.kvp2filterxml(self.parent.kvp, self.parent.context)
-            if tmp_filter is not "":
+            tmp_filter = opensearch.kvp2filterxml(self.parent.kvp, self.parent.context,
+                                                  self.parent.profiles)
+            if tmp_filter != "":
                 self.parent.kvp['constraint'] = tmp_filter
                 LOGGER.debug('OpenSearch Geo/Time parameters to Filter: %s.', self.parent.kvp['constraint'])
 
@@ -808,7 +808,7 @@ class Csw2(object):
             else:
                 self.parent.kvp['sortby']['order'] = 'ASC'
 
-        if 'startposition' not in self.parent.kvp:
+        if 'startposition' not in self.parent.kvp or not self.parent.kvp['startposition']:
             self.parent.kvp['startposition'] = 1
 
         # query repository
