@@ -1,11 +1,11 @@
-# -*- coding: iso-8859-15 -*-
+# -*- coding: utf-8 -*-
 # =================================================================
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
-#                Angelos Tzotsos <tzotsos@gmail.com>
+#          Angelos Tzotsos <tzotsos@gmail.com>
 #
-# Copyright (c) 2011 Tom Kralidis
-# Copyright (c) 2011 Angelos Tzotsos
+# Copyright (c) 2015 Tom Kralidis
+# Copyright (c) 2015 Angelos Tzotsos
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -31,8 +31,8 @@
 # =================================================================
 
 import os
-from lxml import etree
-from pycsw import config, util
+from pycsw.core import config, util
+from pycsw.core.etree import etree
 from pycsw.plugins.profiles import profile
 
 CODELIST = 'http://www.isotc211.org/2005/resources/Codelist/gmxCodelists.xml'
@@ -351,9 +351,12 @@ class APISO(profile.Profile):
         schemaLanguage='XMLSCHEMA', targetNamespace=self.namespace,
         parentSchema='gmd.xsd')
 
-        schema = etree.parse(os.path.join(self.context.pycsw_home,
-                 'plugins', 'profiles', 'apiso', 'schemas', 'ogc', 'iso',
-                 '19139', '20060504', 'gmd', 'identification.xsd')).getroot()
+        schema_file = os.path.join(self.context.pycsw_home, 'plugins',
+                                   'profiles', 'apiso', 'schemas', 'ogc',
+                                   'iso', '19139', '20060504', 'gmd',
+                                   'identification.xsd')
+
+        schema = etree.parse(schema_file, self.context.parser).getroot()
 
         node1.append(schema)
 
@@ -362,9 +365,12 @@ class APISO(profile.Profile):
         schemaLanguage='XMLSCHEMA', targetNamespace=self.namespace,
         parentSchema='gmd.xsd')
 
-        schema = etree.parse(os.path.join(self.context.pycsw_home, 'plugins',
-                 'profiles', 'apiso', 'schemas', 'ogc', 'iso', '19139',
-                 '20060504', 'srv', 'serviceMetadata.xsd')).getroot()
+        schema_file = os.path.join(self.context.pycsw_home, 'plugins',
+                                   'profiles', 'apiso', 'schemas', 'ogc',
+                                   'iso', '19139', '20060504', 'srv',
+                                   'serviceMetadata.xsd')
+
+        schema = etree.parse(schema_file, self.context.parser).getroot()
 
         node2.append(schema)
 
@@ -374,15 +380,26 @@ class APISO(profile.Profile):
         '''Perform extra profile specific checks in the GetDomain request'''
         return None
 
-    def write_record(self, result, esn, outputschema, queryables):
+    def write_record(self, result, esn, outputschema, queryables, caps=None):
         ''' Return csw:SearchResults child as lxml.etree.Element '''
         typename = util.getqattr(result, self.context.md_core_model['mappings']['pycsw:Typename'])
-        if (esn == 'full' and typename == 'gmd:MD_Metadata'):
-            # dump record as is and exit
-            return etree.fromstring(util.getqattr(result, self.context.md_core_model['mappings']['pycsw:XML']))
+        is_iso_anyway = False
 
-        if typename == 'csw:Record':  # transform csw:Record -> gmd:MD_Metadata model mappings
-            util.transform_mappings(queryables, self.repository['mappings']['csw:Record'])
+        xml_blob = util.getqattr(result, self.context.md_core_model['mappings']['pycsw:XML'])
+
+        #xml_blob_decoded = bytes.fromhex(xml_blob[2:]).decode('utf-8')
+
+        if isinstance(xml_blob, bytes):
+            iso_string = b'<gmd:MD_Metadata>'
+        else:
+            iso_string = '<gmd:MD_Metadata>'
+
+        if caps is None and xml_blob is not None and xml_blob.startswith(iso_string):
+            is_iso_anyway = True
+
+        if (esn == 'full' and (typename == 'gmd:MD_Metadata' or is_iso_anyway)):
+            # dump record as is and exit
+            return etree.fromstring(xml_blob, self.context.parser)
 
         node = etree.Element(util.nspath_eval('gmd:MD_Metadata', self.namespaces))
         node.attrib[util.nspath_eval('xsi:schemaLocation', self.context.namespaces)] = \
@@ -404,7 +421,7 @@ class APISO(profile.Profile):
         # hierarchyLevel
         mtype = util.getqattr(result, queryables['apiso:Type']['dbcol']) or None
 
-        if mtype is not None: 
+        if mtype is not None:
             if mtype == 'http://purl.org/dc/dcmitype/Dataset':
                 mtype = 'dataset'
             hierarchy = etree.SubElement(node, util.nspath_eval('gmd:hierarchyLevel', self.namespaces))
@@ -412,12 +429,76 @@ class APISO(profile.Profile):
 
         if esn in ['summary', 'full']:
             # contact
-            val = util.getqattr(result, queryables['apiso:OrganisationName']['dbcol'])
             contact = etree.SubElement(node, util.nspath_eval('gmd:contact', self.namespaces))
-            if val:
+            if caps is not None:
                 CI_resp = etree.SubElement(contact, util.nspath_eval('gmd:CI_ResponsibleParty', self.namespaces))
-                org_name = etree.SubElement(CI_resp, util.nspath_eval('gmd:organisationName', self.namespaces))
-                etree.SubElement(org_name, util.nspath_eval('gco:CharacterString', self.namespaces)).text = val
+                if hasattr(caps.provider.contact, 'name'):
+                    ind_name = etree.SubElement(CI_resp, util.nspath_eval('gmd:individualName', self.namespaces))
+                    etree.SubElement(ind_name, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.name
+                if hasattr(caps.provider.contact, 'organization'):
+                    if caps.provider.contact.organization is not None:
+                        org_val = caps.provider.contact.organization
+                    else:
+                        org_val = caps.provider.name
+                    org_name = etree.SubElement(CI_resp, util.nspath_eval('gmd:organisationName', self.namespaces))
+                    etree.SubElement(org_name, util.nspath_eval('gco:CharacterString', self.namespaces)).text = org_val
+                if hasattr(caps.provider.contact, 'position'):
+                    pos_name = etree.SubElement(CI_resp, util.nspath_eval('gmd:positionName', self.namespaces))
+                    etree.SubElement(pos_name, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.position
+                contact_info = etree.SubElement(CI_resp, util.nspath_eval('gmd:contactInfo', self.namespaces))
+                ci_contact = etree.SubElement(contact_info, util.nspath_eval('gmd:CI_Contact', self.namespaces))
+                if hasattr(caps.provider.contact, 'phone'):
+                    phone = etree.SubElement(ci_contact, util.nspath_eval('gmd:phone', self.namespaces))
+                    ci_phone = etree.SubElement(phone, util.nspath_eval('gmd:CI_Telephone', self.namespaces))
+                    voice = etree.SubElement(ci_phone, util.nspath_eval('gmd:voice', self.namespaces))
+                    etree.SubElement(voice, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.phone
+                    if hasattr(caps.provider.contact, 'fax'):
+                        fax = etree.SubElement(ci_phone, util.nspath_eval('gmd:facsimile', self.namespaces))
+                        etree.SubElement(fax, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.fax
+                address = etree.SubElement(ci_contact, util.nspath_eval('gmd:address', self.namespaces))
+                ci_address = etree.SubElement(address, util.nspath_eval('gmd:CI_Address', self.namespaces))
+                if hasattr(caps.provider.contact, 'address'):
+                    delivery_point = etree.SubElement(ci_address, util.nspath_eval('gmd:deliveryPoint', self.namespaces))
+                    etree.SubElement(delivery_point, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.address
+                if hasattr(caps.provider.contact, 'city'):
+                    city = etree.SubElement(ci_address, util.nspath_eval('gmd:city', self.namespaces))
+                    etree.SubElement(city, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.city
+                if hasattr(caps.provider.contact, 'region'):
+                    admin_area = etree.SubElement(ci_address, util.nspath_eval('gmd:administrativeArea', self.namespaces))
+                    etree.SubElement(admin_area, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.region
+                if hasattr(caps.provider.contact, 'postcode'):
+                    postal_code = etree.SubElement(ci_address, util.nspath_eval('gmd:postalCode', self.namespaces))
+                    etree.SubElement(postal_code, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.postcode
+                if hasattr(caps.provider.contact, 'country'):
+                    country = etree.SubElement(ci_address, util.nspath_eval('gmd:country', self.namespaces))
+                    etree.SubElement(country, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.country
+                if hasattr(caps.provider.contact, 'email'):
+                    email = etree.SubElement(ci_address, util.nspath_eval('gmd:electronicMailAddress', self.namespaces))
+                    etree.SubElement(email, util.nspath_eval('gco:CharacterString', self.namespaces)).text = caps.provider.contact.email
+
+                contact_url = None
+                if hasattr(caps.provider, 'url'):
+                    contact_url = caps.provider.url
+                if hasattr(caps.provider.contact, 'url') and caps.provider.contact.url is not None:
+                    contact_url = caps.provider.contact.url
+
+                if contact_url is not None:
+                    online_resource = etree.SubElement(ci_contact, util.nspath_eval('gmd:onlineResource', self.namespaces))
+                    gmd_linkage = etree.SubElement(online_resource, util.nspath_eval('gmd:linkage', self.namespaces))
+                    etree.SubElement(gmd_linkage, util.nspath_eval('gmd:URL', self.namespaces)).text = contact_url
+
+                if hasattr(caps.provider.contact, 'role'):
+                    role = etree.SubElement(CI_resp, util.nspath_eval('gmd:role', self.namespaces))
+                    role_val = caps.provider.contact.role
+                    if role_val is None:
+                        role_val = 'pointOfContact'
+                    etree.SubElement(role, util.nspath_eval('gmd:CI_RoleCode', self.namespaces), codeListValue=role_val, codeList='%s#CI_RoleCode' % CODELIST).text = role_val
+            else:
+                val = util.getqattr(result, queryables['apiso:OrganisationName']['dbcol'])
+                if val:
+                    CI_resp = etree.SubElement(contact, util.nspath_eval('gmd:CI_ResponsibleParty', self.namespaces))
+                    org_name = etree.SubElement(CI_resp, util.nspath_eval('gmd:organisationName', self.namespaces))
+                    etree.SubElement(org_name, util.nspath_eval('gco:CharacterString', self.namespaces)).text = val
 
             # date
             val = util.getqattr(result, queryables['apiso:Modified']['dbcol'])
@@ -451,7 +532,7 @@ class APISO(profile.Profile):
            restagname = 'srv:SV_ServiceIdentification'
         else:
            restagname = 'gmd:MD_DataIdentification'
-          
+
         resident = etree.SubElement(identification, util.nspath_eval(restagname, self.namespaces), id=idval)
         tmp2 = etree.SubElement(resident, util.nspath_eval('gmd:citation', self.namespaces))
         tmp3 = etree.SubElement(tmp2, util.nspath_eval('gmd:CI_Citation', self.namespaces))
@@ -528,7 +609,7 @@ class APISO(profile.Profile):
             if kw is not None:
                 srv_keywords = etree.SubElement(resident, util.nspath_eval('srv:keywords', self.namespaces))
                 srv_keywords.append(write_keywords(kw, self.namespaces))
-                 
+
             if bboxel is not None:
                 bboxel.tag = util.nspath_eval('srv:extent', self.namespaces)
                 resident.append(bboxel)
@@ -562,10 +643,10 @@ class APISO(profile.Profile):
                         etree.SubElement(tmp2, util.nspath_eval('gco:CharacterString', self.namespaces)).text = i
 
                         tmp3 = etree.SubElement(tmp, util.nspath_eval('srv:DCP', self.namespaces))
-                        etree.SubElement(tmp3, util.nspath_eval('srv:DCPList', self.namespaces), codeList='%s#DCPList' % CODELIST, codeListValue='HTTPGet').text = 'HTTPGet' 
+                        etree.SubElement(tmp3, util.nspath_eval('srv:DCPList', self.namespaces), codeList='%s#DCPList' % CODELIST, codeListValue='HTTPGet').text = 'HTTPGet'
 
                         tmp4 = etree.SubElement(tmp, util.nspath_eval('srv:DCP', self.namespaces))
-                        etree.SubElement(tmp4, util.nspath_eval('srv:DCPList', self.namespaces), codeList='%s#DCPList' % CODELIST, codeListValue='HTTPPost').text = 'HTTPPost' 
+                        etree.SubElement(tmp4, util.nspath_eval('srv:DCPList', self.namespaces), codeList='%s#DCPList' % CODELIST, codeListValue='HTTPPost').text = 'HTTPPost'
 
                         connectpoint = etree.SubElement(tmp, util.nspath_eval('srv:connectPoint', self.namespaces))
                         onlineres = etree.SubElement(connectpoint, util.nspath_eval('gmd:CI_OnlineResource', self.namespaces))
@@ -657,7 +738,7 @@ def _get_resource_opname(operations):
     return None
 
 def _write_codelist_element(codelist_element, codelist_value, nsmap):
-    namespace, codelist = codelist_element.split(':')    
+    namespace, codelist = codelist_element.split(':')
 
     element = etree.Element(util.nspath_eval(codelist_element, nsmap),
     codeSpace=CODESPACE, codeList='%s#%s' % (CODELIST, codelist),
@@ -666,4 +747,3 @@ def _write_codelist_element(codelist_element, codelist_value, nsmap):
     element.text = codelist_value
 
     return element
-
